;       ----------------------------------------------------

;    9999-in-1 - version 0.2
;    Copyright 2010 Don Miller
;    For more information, visit: http://www.no-carrier.com

;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

;       ----------------------------------------------------

NewButtons = $c1
OldButtons = $c2
JustPressed = $c3

PRGNumber = $c4
OldPRG = $c5

;       ----------------------------------------------------

        .ORG $7ff0
Header:                         ; 16 byte .NES header (iNES format)
	.DB "NES", $1a
	.DB $08                 ; size of PRG ROM in 16kb units = 128k = 4 * 32k PRG's (NROM)
	.DB $04			; size of CHR ROM in 8kb units = 32k = 4 * 8k CHR's (NROM)
	.DB #%00100000		; mapper 66 - GNROM
	.DB #%01000000		; mapper 66 - GNROM
        .DB $00
        .DB $00
        .DB $00
        .DB $00
        .DB $00
        .DB $00
        .DB $00
        .DB $00

;       ----------------------------------------------------

        .org $8000

Reset:                          ; reset routine
        SEI
        CLD
	LDX #$00
	STX $2000
	STX $2001
	DEX
	TXS
  	LDX #0
  	TXA
ClearMemory:
	STA 0, X
	STA $100, X
	STA $200, X
	STA $300, X
	STA $400, X
	STA $500, X
	STA $600, X
	STA $700, X
        STA $800, X
        STA $900, X
        INX
	BNE ClearMemory

;       ----------------------------------------------------

	LDX #$02                ; warm up
WarmUp:
	BIT $2002
	BPL WarmUp
	DEX
	BNE WarmUp

       	LDA #$3F
	STA $2006
	LDA #$00
	STA $2006
        TAX
LoadPal:                        ; load palette
        LDA palette, x
        STA $2007
        INX
        CPX #$20
        BNE LoadPal

	LDA #$20
	STA $2006
	LDA #$00
	STA $2006

	LDY #$04                ; clear nametables
ClearName:
	LDX #$00
	LDA #$00
PPULoop:
	STA $2007
	DEX
	BNE PPULoop

	DEY
	BNE ClearName

;       ----------------------------------------------------

        LDA #$00                ; set up variables
        STA PRGNumber

;       ----------------------------------------------------

        ldx #0                  ; copies the bank switching routine to RAM -
copy_bank_code:                 ; this is needed, as the entire 32k bank is
        lda bank_code,x         ; switched at one time, so the code needs to
        sta $300,x              ; reside somewhere outside of the bank itself
        inx
        cpx #12
        bne copy_bank_code

;       ----------------------------------------------------

        JSR DrawScreen          ; draw initial nametable
        JSR InitSprites
        JSR Vblank              ; turn on screen

;       ----------------------------------------------------

InfLoop:                        ; loop forever, program now controlled by NMI routine

        JMP InfLoop

;       ----------------------------------------------------

DrawScreen:

        LDA #<pic              ; load low byte of first picture
        STA $10
        LDA #>pic              ; load high byte of first picture
        STA $11

   	LDA #$20                ; set to beginning of first nametable
    	STA $2006
    	LDA #$00
    	STA $2006

        LDY #$00
        LDX #$04

NameLoop:                       ; loop to draw entire nametable
        LDA ($10),y
        STA $2007
        INY
        BNE NameLoop
        INC $11
        DEX
        BNE NameLoop

        RTS

;       ----------------------------------------------------

InitSprites:
      LDA #$ff
      LDX #$00
ClearSprites:
      STA $500, x
      INX
      BNE ClearSprites

      LDA #$00
      STA $2003                 ; set the low byte (00) of the RAM address
      LDA #$05
      STA $4014                 ; set the high byte (05) of the RAM address

LoadSprites:
      LDX #$00
LoadSpritesLoop:
      LDA sprites, x            ; load data from address
      STA $0500, x              ; store into RAM address
      INX
      CPX #4
      BNE LoadSpritesLoop
      RTS

sprites:
           ;vert tile attr horiz
        .db $17, $2D, $00, $17  ; sprite 0

;       ----------------------------------------------------

UpdateSprites:
        LDA #$00
        STA $2003
        LDA #$05
        STA $4014
        RTS

;       ----------------------------------------------------

ChangeSprite:
        LDX PRGNumber
        LDA Line,x
        STA $0500
        RTS

Line:
        .db $17,$27,$37

;       ----------------------------------------------------

Vblank:                         ; turn on the screen and start the party
	BIT $2002
	BPL Vblank

        LDX #$00
        STX $2005
        STX $2005

	LDA #%10001000
	STA $2000
        LDA #%00011110
	STA $2001

        RTS

;       ----------------------------------------------------

ControllerTest:

        LDA NewButtons
	STA OldButtons

        LDX #$00
	LDA #$01		; strobe joypad
	STA $4016
	LDA #$00
	STA $4016
ConLoop:
	LDA $4016	       ; check the state of each button
	LSR
	ROR NewButtons
        INX
        CPX #$08
        bne ConLoop

	LDA OldButtons         ; invert bits
	EOR #$FF
	AND NewButtons
	STA JustPressed

	LDA PRGNumber          ; save old PRG number for later compare
	STA OldPRG

CheckDown:
	LDA #%00100000
	AND JustPressed
	BEQ CheckUp

	INC PRGNumber          ; increment PRG number here
        LDA PRGNumber
	CMP #3	               ; equal to total # of PRGs +1, starting from 0
	BNE CheckUp
	LDA #0
	STA PRGNumber

CheckUp:
	LDA #%00010000
	AND JustPressed
	BEQ CheckStart

        DEC PRGNumber          ; decrement PRG number here
        BPL CheckStart
	LDA #2	               ; equal to total # of PRGs, starting from 0
	STA PRGNumber

CheckStart:
	LDA #%00001000
	AND JustPressed
	BEQ EndCheck

        jmp $300               ; jump bankswitch routine in RAM, set up during init

EndCheck:
	LDA PRGNumber          ; has PRG number changed? if so, move sprite
	CMP OldPRG
	BEQ CheckOver
	JSR ChangeSprite

CheckOver:

        RTS

;       ----------------------------------------------------

NMI:
        JSR UpdateSprites
        JSR ControllerTest      ; check for user input
        RTI
IRQ:
        RTI

;       ----------------------------------------------------

palette:                        ; palette data
        .byte $0F,$10,$20,$30,$0F,$10,$20,$30,$0F,$10,$20,$30,$0F,$10,$20,$30
        .byte $0F,$10,$20,$30,$0F,$10,$20,$30,$0F,$10,$20,$30,$0F,$10,$20,$30

;       ----------------------------------------------------

pic:
        .INCBIN "screen.nam"

;       ----------------------------------------------------

bank_code:                      ; this code is copied to RAM @ $300 during init
.byte $A6, $C4                  ; LDX PRGNumber
.byte $BD, $F0, $FF             ; LDA banktable,X
.byte $9D, $F0, $FF             ; STA banktable,X
.byte $6C, $FC, $FF             ; JMP ($FFFC)

;       ----------------------------------------------------

        .org $fff0              ; the bank data is stored here to avoid bus conflicts
banktable:                      ; write to this table to switch banks (PRG + CHR).
        .byte $11, $22, $33     ; more @ http://wiki.nesdev.com/w/index.php/Bus_conflict

;       ----------------------------------------------------

	.ORG $fffa              ; vectors
	.DW NMI
	.DW Reset
	.DW IRQ

;       ----------------------------------------------------

        .INCBIN "1.prg"         ; PRG data for ROM's 2, 3, and 4
        .INCBIN "2.prg"
        .INCBIN "3.prg"

        .INCBIN "0.chr"         ; CHR data for all ROM's
        .INCBIN "1.chr"
        .INCBIN "2.chr"
        .INCBIN "3.chr"